<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:dc="http://purl.org/dc/elements/1.1/"
	xmlns:ws="http://www.ebenoit.info/data"
	xmlns:wsf="http://www.ebenoit.info/ext">


	<!-- *********************************************************************************************** -->
	<!-- * RSS FEEDS BOX                                                                               * -->
	<!-- *********************************************************************************************** -->


	<!-- Generate the feeds box
	   -
	   - This template generates a side box for the RSS feeds and calls the appropriate generator
	   - template.
	  -->
	<xsl:template name="generate-feeds-box">
		<xsl:call-template name="side-box-template">
			<xsl:with-param name="id">feeds-box</xsl:with-param>
			<xsl:with-param name="title">
				<xsl:call-template name="get-i18n-string">
					<xsl:with-param name="id">rss-feeds</xsl:with-param>
				</xsl:call-template>
			</xsl:with-param>
			<xsl:with-param name="contents">
				<xsl:call-template name="generate-feeds" />
			</xsl:with-param>
		</xsl:call-template>
	</xsl:template>


	<!-- Generate the feeds display
	   -
	   - This template generates the contents of the side box for the RSS feeds.
	  -->
	<xsl:template name="generate-feeds">
		<xsl:choose>

			<xsl:when test="$d/ws:feeds/ws:feed">
				<xsl:for-each select="$d/ws:feeds/ws:feed">
					<xsl:call-template name="generate-feed" />
				</xsl:for-each>
			</xsl:when>

			<xsl:otherwise>
				<div class="line-content">
					<xsl:call-template name="get-i18n-string">
						<xsl:with-param name="id">no-feeds</xsl:with-param>
					</xsl:call-template>
				</div>
			</xsl:otherwise>

		</xsl:choose>
	</xsl:template>


	<!-- Generate a single feed's display
	   -
	   - This template generates the contents that correspond to a single RSS feed.
	  -->
	<xsl:template name="generate-feed">
		<xsl:variable name="feed" select="document( @from )" />
		<xsl:variable name="hide-links" select="@hide-links and @hide-links = '1'" />

		<xsl:for-each select="$feed/rss/channel">
			<div class="rss-feed">
				<div class="feed-title">
					<xsl:value-of select="title" />
				</div>
				<div class="feed-container">
					<xsl:if test="description">
						<div class="feed-description">
							<xsl:value-of select="description" />
						</div>
					</xsl:if>
					<div class="feed-contents">
						<xsl:for-each select="item[position() &lt;= 5]">
							<xsl:call-template name="generate-feed-item">
								<xsl:with-param name="hide-links" select="$hide-links" />
							</xsl:call-template>
						</xsl:for-each>
					</div>
				</div>
			</div>
		</xsl:for-each>

	</xsl:template>


	<!-- Generate an RSS item display
	   -
	   - This template generates the contents that correspond to a single RSS feed item.
	  -->
	<xsl:template name="generate-feed-item">
		<xsl:param name="hide-links" />

		<div class="feed-item">
			<div class="feed-item-title">
				<xsl:choose>
					<xsl:when test="$hide-links or not(link)">
						<xsl:value-of select="title" disable-output-escaping="no" />
					</xsl:when>
					<xsl:otherwise>
						<a>
							<xsl:attribute name="href">
								<xsl:value-of select="link" />
							</xsl:attribute>
							<xsl:value-of select="title" disable-output-escaping="yes" />
						</a>
					</xsl:otherwise>
				</xsl:choose>
			</div>
			<div class="feed-item-date">
				<xsl:if test="dc:creator">
					<xsl:value-of select="dc:creator" /> /
				</xsl:if>
				<xsl:value-of select="pubDate" />
			</div>
		</div>
	</xsl:template>


</xsl:stylesheet>

