# -*- coding: utf-8 -*-

"""

PySSE v0.2.0

Copyright (C) 2009 Emmanuel Benoît <ebenoit@ebenoit.info>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.

"""

from mod_python import apache
from os.path import dirname
from lxml import etree
import re


def get_languages( dataFile ):

	lFragments = dataFile.xpath( '/ws:data/ws:contents' ,
			namespaces = { 'ws': 'http://www.ebenoit.info/data' } )

	languages = [ node.get( 'lang' ) for node in lFragments ]
	if not languages:
		languages = [ 'en' ]

	return languages


def guess_language( acceptString , languages ):
	acceptString = re.sub( r'[^a-zA-Z0-9\.\-;,=]' , '' , acceptString )
	if len( acceptString ) == 0:
		return languages[ 0 ]

	# Build a list using the different weights as keys
	accepted = { }
	aList = acceptString.split( ',' )

	for lString in aList:

		if lString == '':
			continue

		lData = lString.split( ';q=' )
		if len( lData ) == 1:
			lData.append( '1.0' )

		lStr = lData[ 0 ]
		lWeight = float( lData[ 1 ] ) * 1000

		if lWeight in accepted:
			accepted[ lWeight ].append( lStr )
		else:
			accepted[ lWeight ] = [ lStr ]

	# Iterate for all languages in the weight-specific order
	weights = accepted.keys( );
	weights.sort( );
	for lWeight in reversed( weights ) :
		for lStr in accepted[ lWeight ]:
			if lStr in languages:
				return lStr

	return languages[ 0 ]



def handler( req ):

	# Find the script's base URI and physical path
	sPath = req.parsed_uri[ apache.URI_PATH ]
	if sPath.find( 'index.py' ) == -1:
		sPath = sPath + 'index.py'
	if req.path_info:
		sPath = sPath[ 0 : -len( req.path_info ) ]
	basePath = dirname( sPath )
	if basePath[ len( basePath ) - 1 ] != '/' :
		basePath = basePath + '/'
	dataPath = dirname( req.finfo[apache.FINFO_FNAME] ) + '/data/'

	# Extract requested language and page from the path
	lang = ""
	page = "main"
	if req.path_info:
		splitPath = req.path_info[ 1 : ].split( '/' )
		if len( splitPath ) == 1:
			page = splitPath.pop(0)
		elif len( splitPath ) >= 2:
			lang = splitPath.pop(0)
			page = splitPath.pop(0)

	# Check if the page name is valid
	vPageExp = re.compile( '^[a-z]+$' , re.IGNORECASE )
	if not vPageExp.match( page ):
		page = "main"

	# Some useful variables for handling namespaces
	xhtml_ns = 'http://www.w3.org/1999/xhtml'
	xhtml = '{%s}' % xhtml_ns
	ws_ns = 'http://www.ebenoit.info/data'
	ws = '{%s}' % ws_ns
	nsmap = { None : xhtml_ns , 'ws' : ws_ns }

	# Define the extension function
	dataFile = None
	def page_data_container( d_func ):
		def f( *args ):
			try:
				l = d_func( *args )
				x = etree.Element( ws + 'data' , nsmap = nsmap )
				x.extend( l )
				return [ x ]
			except Exception, e:
				hdr = etree.Element( xhtml + 'h2' , nsmap = nsmap )
				hdr.text = 'Internal error'
				p = etree.Element( xhtml + 'p' , nsmap = nsmap )
				p.text = str( e )
				return [ hdr , p ]
		return f

	@page_data_container
	def get_page_data( ignore_this , page_id , lang_id ):

		# Find the page's record
		ns = { 'ws' : ws_ns }
		xpath_expr = ( '/ws:data/ws:contents[ @lang = $lang ]'
					   + '/ws:pages/ws:page[ @id = $pid ]' )
		page_lst = dataFile.xpath( xpath_expr ,
									lang = lang_id ,
									pid = page_id ,
									namespaces = ns )
		if len( page_lst ) != 1:
			hdr = etree.Element( xhtml + 'h2' , nsmap = nsmap )
			hdr.text = 'Error for page ' + page_id + ' in language ' + lang_id
			p = etree.Element( xhtml + 'p' , nsmap = nsmap )
			p.text = 'The page could not be found in the site\'s configuration.'
			return [ hdr , p ]
		page = page_lst[ 0 ]

		# Does it have a separate data file?
		if 'datafile' not in page.attrib:
			# Nope, return the tag's contents
			return list( page )

		# Get the parameters that determine the path to the page's data file
		file_path = dataPath
		xpath_expr = '/ws:data/ws:page-data'
		r = dataFile.xpath( xpath_expr , namespaces = ns )
		if len( r ) == 1:
			file_path += r[ 0 ].text + '/'
		xpath_expr = '/ws:data/ws:contents[ @lang = $lang ]/@language-path'
		r = dataFile.xpath( xpath_expr , lang = lang_id , namespaces = ns )
		if len( r ) == 1:
			file_path += lang_id + '/'
		else:
			xpath_expr = '/ws:data/ws:contents[ @lang = $lang ]/@base-path'
			r = dataFile.xpath( xpath_expr , lang = lang_id ,
								namespaces = ns )
			if len( r ) == 1:
				file_path += r[ 0 ].text + '/'
		file_path += page.attrib[ 'datafile' ] + '.xml'

		# Get the data from the file
		try:
			page_data = etree.parse( file_path )
		except Exception , e:
			hdr = etree.Element( xhtml + 'h2' , nsmap = nsmap )
			hdr.text = 'Error for page ' + page_id + ' in language ' + lang_id
			p = etree.Element( xhtml + 'p' , nsmap = nsmap )
			p.text = str( e )
			return [ hdr , p ]

		return list( page_data.getroot() )

	# Register it
	ext_ns = etree.FunctionNamespace( 'http://www.ebenoit.info/ext' )
	ext_ns[ 'get-page-data' ] = get_page_data

	# Read the data file
	dataFile = etree.parse( dataPath + 'site.xml' )

	# Find out which languages are supported
	languages = get_languages( dataFile )

	# Check if the selected language is valid
	if lang != '' and lang not in languages:
		lang = ''

	# If there is no specified language, try to guess it
	# according to Accept-Language
	if lang == '':
		if req.headers_in.has_key( 'Accept-Language' ):
			acceptString = req.headers_in[ 'Accept-Language' ]
			lang = guess_language( acceptString , languages )
		else:
			lang = languages[ 0 ]

	# Load the XSLT stylesheet
	transform = etree.XSLT( etree.parse( dataPath + 'site.xsl' ) )

	# Check if the client accepts application/xhtml+xml
	if req.headers_in.has_key( 'Accept' ):
		accept = re.split( '[;,]' , req.headers_in[ 'Accept' ].lower() )
		if 'application/xhtml+xml' in accept:
			req.content_type = 'application/xhtml+xml'
		else:
			req.content_type = 'text/html'
	else:
		req.content_type = 'text/html'

	req.write( str( transform( dataFile ,
			reqPage = "'" + page + "'" ,
			reqLang = "'" + lang + "'" ,
			webRoot = "'" + basePath + "'" ,
			scriptPath = "'" + sPath + "'" ) ) )

	return apache.OK
